@@ -302,7 +302,43 @@ bool CallManager::command( const QString& cmd )
     return true;
 }
 
-void CallManager::startIncomingCall( const QString& number, bool dialBack )
+void CallManager::emitRing(const CallInfo &info)
+{
+    QString str;
+
+    // Annouce the incoming call using GSM 27.007 notifications.
+    if ( info.state == CallState_Waiting ) {
+        str = "+CCWA: " + QAtUtils::encodeNumber( info.number ) + ",1,";
+
+        if (info.number.isNull())
+            str += "2";
+        else if (info.number.isEmpty())
+            str += "1";
+        else
+            str += "0";
+    } else {
+        str = "+CRING: VOICE";
+
+        if (info.number.isNull())
+            ;
+	else if (info.number.isEmpty())
+            str += "\\n\\n+CLIP: " + QAtUtils::encodeNumber( info.number ) + ",,,,1";
+        else
+            str += "\\n\\n+CLIP: " + QAtUtils::encodeNumber( info.number ) + ",,,,0";
+
+        if (info.name.isNull())
+            ;
+	else if (info.name.isEmpty())
+            str += "\\n\\n+CNAP: \"" + info.name + "\",1";
+        else
+            str += "\\n\\n+CNAP: \"" + info.name + "\",0";
+    }
+
+    emit unsolicited(str);
+}
+
+void CallManager::startIncomingCall( const QString& number,
+				const QString& name, bool dialBack )
 {
     // Bail out if there is already an incoming call.
     if ( idForIncoming() >= 0 ) {
@@ -322,14 +358,10 @@ void CallManager::startIncomingCall( const QString& number, bool dialBack )
     info.number = number;
     info.incoming = true;
     info.dialBack = dialBack;
+    info.name = name;
     callList += info;
 
-    // Annouce the incoming call using GSM 27.007 notifications.
-    if ( info.state == CallState_Waiting ) {
-        emit unsolicited( "+CCWA: " + QAtUtils::encodeNumber( number ) + ",1" );
-    } else {
-        emit unsolicited( "RING\\n\\n+CLIP: " + QAtUtils::encodeNumber( number ) );
-    }
+    emitRing(info);
 
     // Announce the incoming call using Ericsson-style state notifications.
     sendState( info );
@@ -339,9 +371,9 @@ void CallManager::startIncomingCall( const QString& number, bool dialBack )
     ringTimer->start(2000);
 }
 
-void CallManager::startIncomingCall( const QString& number )
+void CallManager::startIncomingCall( const QString& number , const QString& name )
 {
-    startIncomingCall( number, false );
+    startIncomingCall( number, name, false );
 }
 
 void CallManager::hangupAll()
@@ -643,6 +675,17 @@ void CallManager::dialingToConnected()
     // Transition the call to its new state.
     callList[index].state = CallState_Active;
     sendState( callList[index] );
+    // If the dialed number starts with 05123, disconnect the
+    // call after xx seconds, where xx is part of the dial string
+    // as 05123xx
+    if( callList[index].number.startsWith( "05123" ) ) {
+        bool ok;
+        QString temp = callList[index].number;
+        temp = temp.replace( "05123" , "" );
+        int timeout = temp.toInt( &ok, 10 );
+        timeout = ok ? timeout * 1000 : 10000;
+        QTimer::singleShot( timeout, this, SLOT(hangup()) );
+    }
 }
 
 void CallManager::dialingToAlerting()
@@ -659,18 +702,18 @@ void CallManager::dialingToAlerting()
 
 void CallManager::dialBack()
 {
-    startIncomingCall( "1234567", true );
+    startIncomingCall( "1234567", "Alice", true );
 }
 
 void CallManager::dialBackWithHangup5()
 {
-    startIncomingCall( "1234567", true );
+    startIncomingCall( "1234567", "Bob", true );
     hangupTimer->start( 5000 );
 }
 
 void CallManager::dialBackWithHangup4()
 {
-    startIncomingCall( "1234567", true );
+    startIncomingCall( "1234567", "Mallory", true );
     hangupTimer->start( 4000 );
 }
 
@@ -686,6 +729,12 @@ void CallManager::hangupTimeout()
     }
 }
 
+void CallManager::hangup()
+{
+    send ( "NO CARRIER" );
+    hangupConnected();
+}
+
 void CallManager::sendNextRing()
 {
     if ( idForIncoming() >= 0 ) {
@@ -695,7 +744,7 @@ void CallManager::sendNextRing()
         } else {
             for ( int index = 0; index < callList.size(); ++index )
                 if ( callList[index].id == idForIncoming() && callList[index].state == CallState_Incoming )
-                    emit unsolicited( "RING" );
+                    emitRing(callList[index]);
             ringTimer->start(2000);
         }
     }
